<?php
if ( ! defined( 'ABSPATH' ) ) {
	die( 'Stop there, you asshole' );
}
require_once 'etsy-upload.php';
require_once dirname( __FILE__ ) . '/../data/data_overrides.php';
class ETCPF_Product_uploadhoook extends ETCPF_EtsyUpload {

	/*
	 *
	 * defining private property
	 *      set from constructor
	 * */
	private $product;
	private $db;
	private $listing_table;
	private $feed_table;

	/*
	 *
	 * @product is the product id obtained from hook during product update
	 *
	 * */
	function __construct( $product ) {
		parent::__construct();
		$this->product = $product;
		global $wpdb;
		$this->db            = $wpdb;
		$this->listing_table = $this->db->prefix . 'etcpf_listings';
		$this->feed_table    = $this->db->prefix . 'etcpf_feeds';
	}

	/*
	 *
	 * check_if_eligible_toUpload() Checks if the product is already upload to etsy so that update can be done
	 *
	 * wc_get_product woocommerce global function
	 *
	 * getFeedDetails gets the related feed details
	 *
	 * getPreparedData prepares data for uploading to etsy
	 * */
	public function Invoker() {
		$result   = $this->check_if_eligible_toUpload( $this->product );
		$feed     = $this->check_if_eligible_for_new_Upload( $this->product );
		$reupload = true;

		// check if product is newly created but is linked with uploaded category.
		if ( ! isset( $result->listing_id ) && isset( $feed->id ) ) {
			// prepare the listing and write it to listing table.
			$prepared_listing = $this->prepare_individual_listing( $feed->id, $this->product, $result->id );
			if ( ! $prepared_listing ) {
				return;
			}
			// now create $result for newly created product to upload it to etsy.
			$alt_result             = new stdClass();
			$alt_result->feed_id    = $result->feed_id ? $result->feed_id : $feed->id;
			$alt_result->listing_id = null;
			$alt_result->id         = $result->id ? $result->id : $prepared_listing;
			$alt_result->etsy_state = $result->etsy_state ? $result->etsy_state : 'draft';
			$reupload               = false;
			$result                 = $alt_result;
		}

		/**
		 * The product and variation listings has been preapred and saved to respective listing tables if the product is newly created.
		 *
		 * @TODO: Prepare materials for auto upload on next version.
		 * @TODO: Prepare product when reuploading old product in next version.
		 */
		if ( isset( $result ) ) {
			try {
				$productDetails = wc_get_product( $this->product );
				$feed_details   = $this->getFeedDetails( $result->feed_id );
				$preparedData   = $this->getPreparedData( $productDetails, $feed_details, $result );
				if ( ! $productDetails->is_type( 'variable' ) ) {
					if ( isset( $preparedData['use_sale_price'] ) ) {
						if ( $preparedData['use_sale_price'] === true && $preparedData['has_sale_price'] ) {
							$preparedData['price'] = $preparedData['sale_price'];
						} else {
							$preparedData['price'] = $preparedData['regular_price'];
						}
					} else {
						$preparedData['price'] = $preparedData['regular_price'];
					}
					if ( etcpf_get_etsy_settings( 'price_sync' ) == 'no' ) {
						$price = parent::get_listing_price_from_etsy( $result->listing_id );
						if ( $price ) {
							$preparedData['price'] = $price;
						}
					}
				} elseif ( $productDetails->is_type( 'variable' ) && ! $reupload ) {
					$preparedData['price']    = '100';
					$preparedData['quantity'] = '1';
				}
				if ( $preparedData['state'] == false ) {
					$preparedData['state'] = 'draft';
				}
				if ( $feed_details->SHIPPING_TEMPLATE_ID ) {
					$preparedData['shipping_template_id'] = $feed_details->SHIPPING_TEMPLATE_ID;
				}

				$upload     = parent::_CURLFORUPLOAD( $result->listing_id, $reupload, $preparedData, $result->id, false );
				$listing_id = $result->listing_id ? $result->listing_id : $upload;
				if ( $productDetails->is_type( 'variable' ) ) {
					if ( $feed_details->variation_upload_type === 'single' ) {
						$variationarray = $this->testSinglyVariatedFormat( $productDetails->get_id(), $feed_details->id, $feed_details->variation_upload_profile );
					} else {
						$variationarray = $this->prepareVariationForUpload( $productDetails, $productDetails->get_available_variations(), $feed_details, $reupload );
						$variationarray = parent::getpreparedVariationDataforEtsy( $variationarray, $listing_id, false );
					}
					if ( $variationarray ) {
						$upload_results = parent::variationUpload( $variationarray, $listing_id );
						$this->handleImageUpload( $upload, $this->product, $preparedData['image_link'] );
						return $upload_results;

					}
				} else {
					$this->handleImageUpload( $upload, $this->product, $preparedData['image_link'] );
				}
				return $upload;
			} catch ( Exception $e ) {
				$this->recordFailedProduct( $this->product );
			}
		}
		return false;
	}

	public function prepare_individual_listing( $feed_id, $product_id, $stored_listing = null ) {
		// TODO: handle state false and other things
		global $wpdb;
		$table         = $wpdb->prefix . 'etcpf_listings';
		$prepared_data = $this->prepare_listing_data( $product_id, $feed_id );

		$datatobeinserted                            = array(
			'item_id'       => $product_id,
			'title'         => $prepared_data['prepared_data']['title'],
			'has_variation' => $prepared_data['listing_data']->has_variation,
			'feed_id'       => $feed_id,
			'data'          => maybe_serialize( $prepared_data['listing_data'] ),
			'prepared_data' => json_encode( $prepared_data['prepared_data'] ),
			'sku'           => $prepared_data['prepared_data']['sku'] ? $prepared_data['prepared_data']['sku'] : '',
		);
		$datatobeinserted['uploaded']                = 0;
		$datatobeinserted['item_group_id']           = null;
		$datatobeinserted['variation_upload_result'] = null;
		$datatobeinserted['error']                   = 'empty';

		if ( $stored_listing ) {
			if ( $wpdb->update( $table, $datatobeinserted, array( 'id' => $stored_listing ) ) ) {
				return $stored_listing;
			}
		} else {
			if ( $wpdb->insert( $table, $datatobeinserted ) ) {
				return $wpdb->insert_id;
			}
		}
		return false;
	}

	/**
	 * @param $product_id
	 * @param $feed_id
	 * @return mixed
	 */
	public function prepare_listing_data( $product_id, $feed_id ) {
		$productDetails = wc_get_product( $product_id );
		$feed_details   = $this->getFeedDetails( $feed_id );
		$preparedData   = $this->getPreparedData( $productDetails, $feed_details, null );

		$listing_data                 = new stdClass();
		$listing_data->id             = $productDetails->id;
		$listing_data->sku            = $preparedData['sku'];
		$listing_data->title          = $preparedData['title'];
		$listing_data->description    = $preparedData['description'];
		$listing_data->price          = $preparedData['price'];
		$listing_data->quantity       = $preparedData['quantity'];
		$listing_data->state          = $preparedData['state'];
		$listing_data->tags           = $preparedData['tags'];
		$listing_data->etsy_category  = $feed_details->remote_category;
		$listing_data->image_link     = $preparedData['image_link'];
		$listing_data->sale_price     = $preparedData['sale_price'];
		$listing_data->taxonomy_path  = $feed_details->texonomy_path;
		$listing_data->has_variation  = $productDetails->is_type( 'variable' ) ? '1' : '0';
		$listing_data->dimention_unit = get_option( 'woocommerce_dimension_unit' );
		$listing_data->weight_unit    = get_option( 'woocommerce_weight_unit' );
		$listing_data->currency       = get_woocommerce_currency();

		return array(
			'listing_data'  => $listing_data,
			'prepared_data' => $preparedData,
		);
	}

	public function prepareVariationForUpload( $product, $variations, $feed_details, $reupload = true ) {
		$variationArray        = array();
		$wc_product_attributes = $product->get_attributes();
		$wc_attributes         = array();
		foreach ( $wc_product_attributes as $wc_product_attribute_key => $wc_product_attribute_value ) {
			$wc_attribute_name                          = $wc_product_attribute_value['name'];
			$wc_attributes[ $wc_product_attribute_key ] = $wc_attribute_name;
			if ( rawurldecode( $wc_product_attribute_key ) !== $wc_product_attribute_key ) {
				$wc_attributes[ rawurldecode( $wc_product_attribute_key ) ] = $wc_attribute_name;
			}
		}
		if ( is_array( $variations ) && count( $variations ) > 0 ) {
			foreach ( $variations as $key => $variation ) {
				$priceToUpload                                   = $this->getProductPrice( $variation['display_regular_price'], $variation['display_price'] );
				$variationData                                   = new stdClass();
				$variationData->id                               = $variation['variation_id'];
				$variationData->data['id']                       = $variation['variation_id'];
				$variationData->data['sku']                      = $variation['sku'];
				$variationData->data['is_variation']             = true;
				$variationData->data['item_group_id']            = $product->get_id();
				$variationData->data['has_sale_price']           = $variation['display_price'] ? true : false;
				$variationData->data['sale_price']               = $variation['display_price'] ? $variation['display_price'] : 0.00;
				$variationData->data['regular_price']            = $variation['display_regular_price'] ? $variation['display_regular_price'] : 0.00;
				$variationData->data['use_sale_price']           = $this->isSalePriceMappedToPrice();
				$variationData->data['price']                    = $priceToUpload ? $priceToUpload : $product->get_price();
				$variationData->data['variation_images_details'] = $variation['image'];
				$variationData->data['additional_image_link']    = $variation['image']['full_src'];
				$variationData->data['image_link']               = $variation['image']['full_src'];
				$variationData->data['variation_image_id']       = $variation['image_id'];
				$variationData->data['in_stock']                 = $variation['is_in_stock'] ? 'in_stock' : 'out_of_stock';
				$variationData->data['quantity']                 = $variation['max_qty'] ? $variation['max_qty'] : 0;
				$variationData->data['variation_is_active']      = $variation['variation_is_active'];
				$variationData->data['variation_is_visible']     = $variation['variation_is_visible'];

				if ( is_array( $variation['attributes'] ) ) {
					foreach ( $variation['attributes'] as $key => $attr_values ) {
						if ( ! $attr_values ) {
							continue;
						}
						if ( stripos( $key, 'attribute_pa_' ) !== false ) {
							$attr = str_replace( 'attribute_pa_', '', $key );
						} elseif ( stripos( $key, 'attribute_' ) !== false ) {
							$attr = str_replace( 'attribute_', '', $key );
						} else {
							$attr = $key;
						}

						if ( $wc_attributes[ $attr ] ) {
							$attr = $wc_attributes[ $attr ];
						}
						$term_details = get_term_by( 'slug', $attr_values, $key );

						if ( empty( $term_details ) ) {
							if ( $attr_values ) {
								$variationData->data[ $attr ]                         = $attr_values;
								$variationData->data['variation_attributes'][ $attr ] = $attr_values;
							}
						} else {
							$variationData->data[ $attr ]                         = $term_details->name;
							$variationData->data['variation_attributes'][ $attr ] = $term_details->name;
						}
					}
					if ( isset( $variationData->data['variation_attributes'] ) ) {
						$variationData->data['variation_attributes'] = json_encode( $variationData->data['variation_attributes'] );
						$variationData->data                         = (object) $this->ImplementOverride( $variationData->data, $feed_details->feed_overrides, $product );

						$variationArray[] = $variationData;

						if ( $reupload == false ) {
							global $wpdb;
							$table                      = $wpdb->prefix . 'etcpf_listing_variations';
							$datatobeinserted           = array(
								'item_id'   => $variation['variation_id'],
								'parent_id' => $product->get_id(),
								'data'      => maybe_serialize( $variationData ),
								'submitted' => date( 'Y-m-d h:i:sa' ),
								'updated'   => date( 'Y-m-d h:i:sa' ),
							);
							$datatobeinserted['upload'] = 0;
							$wpdb->insert( $table, $datatobeinserted );
						}
						$variationData = null;
					}
				}
			}
			return $variationArray;
		}
		return false;
	}

	private function isSalePriceMappedToPrice() {
		$mappings = get_option( 'etcpf_attribute_user_map_Etsy' );
		if ( $mappings ) {
			$mappings = json_decode( $mappings );
			if ( $mappings->price && $mappings->price !== 'sale_price' ) {
				return false;
			}
		}
		return true;
	}

	private function getProductPrice( $regularPrice, $salePrice ) {
		if ( $this->isSalePriceMappedToPrice() ) {
			if ( $salePrice ) {
				$price = $salePrice;
			} elseif ( $regularPrice ) {
				$price = $regularPrice;
			}
		} else {
			if ( $regularPrice ) {
				$price = $regularPrice;
			}
		}
		if ( $price ) {
			return $price;
		} else {
			return null;
		}
	}


	/*
	 *
	 * Ensures the product provided is already listed in etsy
	 *
	 * @pid product id with which query is performed
	 *
	 *  */
	public function check_if_eligible_toUpload( $pid ) {
		return $this->db->get_row( "SELECT id,listing_id,feed_id, etsy_state FROM {$this->listing_table} WHERE item_id = {$pid}" );
	}

	/*
	 *
	 * Ensures the product provided is in an uploaded category
	 *
	 * @pid product id with which queries are performed
	 *
	 **/
	public function check_if_eligible_for_new_Upload( $pid ) {
		$categories = wc_get_product_term_ids( $pid, 'product_cat' );
		if ( ! $categories || count( $categories ) < 1 ) {
			return false;
		}

		foreach ( $categories as $categoriy ) {
			$feed = $this->db
				->get_row(
					$this->db
						->prepare(
							"SELECT * FROM {$this->feed_table} WHERE category LIKE '%,"
								. $categoriy . ",%' OR category LIKE '%,"
								. $categoriy . "' OR category LIKE '"
								. $categoriy . ",%' OR category LIKE '"
								. $categoriy . "' LIMIT 1"
						)
				);

			if ( $feed ) {
				return $feed;
			}
		}

		return false;
	}

	/*
	 *
	 * @fid feed id to perform the query
	 *
	 * returns the result obtained from query
	 *
	 * */
	public function getFeedDetails( $fid ) {
		return $this->db->get_row( $this->db->prepare( "SELECT * FROM {$this->feed_table} WHERE id=%d", array( $fid ) ) );
	}

	/*
	 * Prepares the data to make upload to etsy
	 *
	 * Communicates with multiple method as
	 *
	 * @product wc_get_product , @feed_details, @listingdata
	 *
	 * returns preparedData
	 * */
	public function getPreparedData( $product, $feed_details, $listingdata ) {
		// Etsy accepts json_encoded array values
		$preparedData = array(
			'title'                => strip_tags( $product->get_name() ),
			'sku'                  => $product->get_sku(),
			'description'          => strip_tags( $product->get_description() ),
			'shipping_template_id' => $this->shipping_template_id,
			'state'                => $this->getValidListingState( $product, $listingdata ),
			'taxonomy_id'          => $this->getTaxonomy( $feed_details, $product ),
			'tags'                 => $this->getValidTags( $product ),
			'who_made'             => etcpf_get_etsy_settings( 'who_made_it' ),
			'is_supply'            => intval( etcpf_get_etsy_settings( 'is_supply' ) ),
			'when_made'            => etcpf_get_etsy_settings( 'when_made' ),
			'recipient'            => 'not_specified',
			'style'                => array( 'Avant garde' ),
			'language'             => etcpf_get_etsy_settings( 'shop_language' ) ? etcpf_get_etsy_settings( 'shop_language' ) : substr( get_locale(), 0, 2 ),
		);

		// If product is variable,
		// we don't need quantity and price to upload in etsy

		$preparedData['regular_price']  = $product->get_regular_price();
		$preparedData['has_sale_price'] = $product->get_sale_price() ? true : false;
		$preparedData['sale_price']     = $preparedData['has_sale_price'] ? $product->get_sale_price() : ( $preparedData['regular_price'] ? $preparedData['regular_price'] : 0 );
		$featured_image_id              = get_post_thumbnail_id( $product->id );
		$featured_image_url             = wp_get_attachment_image_src( $featured_image_id, 'full' ) [0];
		$preparedData['image_link']     = $featured_image_url ? $featured_image_url : '';

		if ( ! $product->is_type( 'variable' ) ) {
			$priceToUpload                  = $this->getProductPrice( $product->get_regular_price(), $product->get_sale_price() );
			$preparedData['use_sale_price'] = $this->isSalePriceMappedToPrice();
			$preparedData['quantity']       = $this->getstockofAproduct( $product );
			$preparedData['price']          = $priceToUpload ? $priceToUpload : $product->get_price();
		}

		if ( etcpf_get_etsy_settings( 'title_sync' ) == 'no' && $listingdata->listing_id ) {
			unset( $preparedData['title'] );
		}

		if ( etcpf_get_etsy_settings( 'description_sync' ) == 'no' && $listingdata->listing_id ) {
			unset( $preparedData['description'] );
		}

		if ( etcpf_get_etsy_settings( 'tags_sync' ) == 'no' && $listingdata->listing_id ) {
			unset( $preparedData['tags'] );
		}
		return $this->ImplementOverride( $preparedData, $feed_details->feed_overrides, $product );
	}

	/*
	 * If product quantity is 0 and item is uploaded to etsy
	 *    make it inactive and send
	 *
	 * @qty: quantity of product
	 *
	 * @listingData: data containing all listed items detail
	 *
	 * */

	public function getValidListingState( $product, $listingData ) {
		$etsyListingstate = $this->checkStatusInEtsy( $listingData->listing_id );
		$defaultState     = etcpf_get_etsy_settings( 'state' );
		if ( $etsyListingstate ) {
			if ( $etsyListingstate === 'removed' ) {
				$state = $defaultState;
			} elseif ( $etsyListingstate === 'active' ) {
				if ( $this->getstockofAproduct( $product ) <= 0 ) {
					$state = 'inactive';
				} else {
					$state = 'active';
				}
			} elseif ( $etsyListingstate === 'draft' ) {
				$state = 'draft';
			} elseif ( $etsyListingstate === 'inactive' ) {
				$state = 'inactive';
			} elseif ( $etsyListingstate === 'edit' ) {
				$state = 'inactive';
			}
			return $state;
		}
		return $defaultState;
	}

	/*
	* return taxonomy id of a etsy category
	* */
	public function getTaxonomy( $details, $product ) {
		if ( $details->feed_type > 0 ) {
			include_once 'ETCPF_Customfeed.php';
			$CustomObj = new ETCPF_Customfeed( 1 );
			$params    = array(
				'filename'   => $details->filename,
				'categories' => $CustomObj->getMappedCategoryOfParticularFeed( $details->id ),
				'products'   => $CustomObj->getProductsOfParticularFeed( $details->id ),
				'feedtype'   => 1,
			);
		}
		$localCategoriesArray = get_the_terms( $product->get_id(), 'product_cat' );
		$localcatString       = '';
		foreach ( $localCategoriesArray as $key => $values ) {
			if ( strlen( $localcatString ) > 3 ) {
				$localcatString .= ',';
			}
			$localcatString .= $values->slug;
		}
		$incaselocalstring = explode( ',', $localcatString );
		if ( isset( $params['categories'][ $localcatString ] ) ) {
			$details->texonomy_path = $params['categories'][ $localcatString ]['taxonomy_path'];
		} elseif ( isset( $params['categories'][ $localcatString ] ) ) {
			$details->texonomy_path = $params['categories'][ $localcatString ]['taxonomy_path'];
		} else {
			foreach ( $incaselocalstring as $key => $value ) {
				if ( isset( $params['categories'][ $value ] ) ) {
					$details->texonomy_path = $params['categories'][ $value ]['taxonomy_path'];
				}
			}
		}
		if ( stripos( $details->texonomy_path, '|' ) ) {
			$current_taxonomy_id = explode( '|', $details->texonomy_path );
		} else {
			$current_taxonomy_id = explode( ',', $details->texonomy_path );
		}
		return end( $current_taxonomy_id );
	}

	/*
	* @returns valid woocommerce tags
	*
	* */
	public function getValidTags( $product ) {
		$t    = '';
		$tags = get_the_terms( $product->get_id(), 'product_tag' );
		if ( is_array( $tags ) && count( $tags ) > 0 ) {
			$tag = array();
			foreach ( $tags as $key => $t ) {
				if ( strlen( $t->name ) <= 20 && preg_match( '/^[a-z\d\-_\s]+$/i', $t->name ) ) {
					$tag[] = $t->name;
				}
			}
			$t = implode( ',', $tag );
		}
		return $t;
	}

	/*
	*
	* Handling externally added attributes
	*
	* @params product data
	*
	* returns formulated data to calling function
	* */
	public function ImplementOverride( $data, $overrides, $product ) {
		if ( $overrides == null ) {
			$overrides = get_option( 'Etsy-etsy-merchant-settings' );
		}
		$invoker = new ETCPF_Data_overrides( $overrides );
		return $invoker->implementOverrides( $data, $product );
	}

	public function recordFailedProduct( $pid ) {
		/* @TODO : record in db */
		return $pid;
	}

	public function getstockofAproduct( $product ) {
		if ( $product->get_stock_quantity() > 0 && ( $product->get_stock_quantity() < 999 ) ) {
			return $product->get_stock_quantity();
		} elseif ( $product->get_stock_quantity() > 0 && ( $product->get_stock_quantity() > 999 ) ) {
			return 999;
		} elseif ( $product->get_manage_stock() === false && $product->get_stock_status() === 'instock' ) {
			return 10;
		} else {
			return 0;
		}
	}

	public function handleImageUpload( $listing_id, $product_id, $featured_image_url ) {
		if ( etcpf_get_etsy_settings( 'images_sync' ) == 'yes' && etcpf_get_etsy_settings( 'wooImage_sync' ) == 'yes' ) {
			parent::delete_listing_images( $listing_id );
			parent::submit_listing_images( $listing_id, 0, $product_id, $featured_image_url, 1, true );
		}
	}

}
