<?php
/***********************************************************
 * Plugin Name: WooCommerce Etsy Integration
 * Plugin URI: www.exportfeed.com
 * Description: Etsy Marketplace Integration for WooCommerce: sync products, orders, and inventory in real-time
 * Author: ExportFeed.com
 * Version: 3.8.6
 * Author URI: www.exportfeed.com
 * License:  GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: etsy-exportfeed-strings
 * Authors: roshanbh
 * WC requires at least: 5.8.0
 * WC tested up to: 6.0.1
 * Note: The "core" folder is shared to the Joomla component.
 * Changes to the core, especially /core/data, should be considered carefully
 * license GNU General Public License version 3 or later; see GPLv3.txt
 ***********************************************************/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
// Exit if accessed directly
require_once ABSPATH . '/wp-admin/includes/plugin.php';
$plugin_version_data = get_plugin_data( __FILE__ );

// sets latest database version
$GLOBALS['etcpf_db_version'] = '1.0';

// current version: used to show version throughout plugin pages.
define( 'ETCPF_PLUGIN_VERSION', $plugin_version_data['Version'] );
define( 'ETCPF_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
define( 'ETCPF_PATH', realpath( dirname( __FILE__ ) ) );
define( 'ETCPF_URL', plugins_url() . '/' . basename( dirname( __FILE__ ) ) . '/' );
// functions to display google_merchant_feeds version and checks for updates.
require_once 'etcpf_functions.php';
require_once 'etsy-export-feeds-information.php';
require_once 'etsy-export-feed-setup.php';
require_once 'core/helpers.php';
require_once 'core/classes/etsy-upload.php';
require_once ABSPATH . 'wp-content/plugins/etsy-plugin/database/upgrade.php';
require_once ABSPATH . 'wp-content/plugins/etsy-plugin/notice/DbUpdateCompletedNotice.php';

// action hook for plugin activation.
register_activation_hook( __FILE__, 'etcpf_activate_plugin' );
register_deactivation_hook( __FILE__, 'etcpf_deactivate_plugin' );

add_action( 'etcpf_plugins_loaded', 'etcpf_activate_plugin' );

//checks for database upgrades
etcpf_db_upgrade();

//check if database is recently updated
$is_db_updated = __get_option( 'etcpf_db_upgraded' );
if ( get_option( 'etcpf_db_upgraded' ) == 'true' ) {
	add_action( 'admin_notices', 'display_etcpf_db_update_completed_notice' );
	update_option( 'etcpf_db_upgraded', 'false' );
	update_option( 'etcpf_db_upgrade_available', 'false' );
};

// action fired after order is completed
$upload = new ETCPF_EtsyUpload();
add_action( 'woocommerce_order_status_completed', array( $upload, 'update_etsy_order_shipment' ) );
// =======================================

global $cp_feed_order, $cp_feed_order_reverse;

// need to know what this cron activity will do.
require_once 'core/classes/cron.php';
require_once 'core/data/feedfolders.php';

if ( get_option( 'et_cpf_feed_order_reverse' ) == '' ) {
	add_option( 'et_cpf_feed_order_reverse', false );
}

if ( get_option( 'et_cp_feed_order' ) == '' ) {
	add_option( 'et_cp_feed_order', 'id' );
}

if ( get_option( 'et_cp_localkey' ) == '' ) {
	add_option( 'et_cp_localkey', 'none' );
}

if ( get_option( 'et_cp_feed_limit' ) == '' ) {
	add_option( 'et_cp_feed_limit', '0' );
}

// seting setup stage phase
if ( get_option( 'etcpf_stage' ) == '' ) {
	add_option( 'etcpf_stage', - 1 );
}

if ( get_option( 'et_cp_feed_count' ) == '' ) {
	add_option( 'et_cp_feed_count', '0' );
}

if ( get_option( 'listing_etsy_et' ) == '' ) {
	add_option( 'listing_etsy_et', 0 );
}
if ( get_option( 'etsy_order_offset' ) == '' ) {
	add_option( 'etsy_order_offset', 0 );
}
add_option( 'etcpf_license_message', 0 );
// ***********************************************************
// cron schedules for Feed Updates
// ***********************************************************
$cron                       = new ETCPF_Cron();
$etcpf_db_upgrade_available = __get_option( 'etcpf_db_upgrade_available' );
require_once 'core/classes/ETCPF_Disable_during_db_upgrade.php';
$db_upgrade = new ETCPF_Disable_during_db_upgrade();

if ( $etcpf_db_upgrade_available == 'true' ) {
	$db_upgrade->etcpf_disable_existing_events();
} else {
	$cron->scheduleetsyFetchProduct();
	if ( etcpf_get_etsy_settings( 'feed_update_interval' ) ) {
		$cron->etsyFeedUpdateCron();
	}

	if ( etcpf_get_etsy_settings( 'order_fetch_interval' ) ) {
		$cron->scheduleetsyOrder();
	}
	if ( etcpf_get_etsy_settings( 'feed_submission_interval' ) ) {
		$cron->scheduleetsyUpload();
	}
}
// ***********************************************************
// Update Feeds (Cron)
// 2014-05-09 Changed to now update all feeds... not just Etsy Feeds
// ***********************************************************

add_action( 'update_etsyfeeds_hook', 'etcpf_update_all_abstract' );
add_filter( 'auto_feed_submission_hook', 'etcpf_auto_feed_submission' );
add_filter( 'auto_fetch_product_hook', 'etcpf_auto_fetch_product' );
add_filter( 'auto_etsy_order_hook', 'etcpf_auto_etsy_order' );
function etcpf_get_gif_loader( $class, $style = array(), $gif = false, $original = false, $ripple = 'ripple.gif', $id = '' ) {
	$css = '';
	if ( count( $style ) > 0 ) {
		foreach ( $style as $key => $value ) {
			$css .= $key . ':' . $value . ';';
		}
	}
	$dimension = ' height="25" wdth="30" ';
	if ( $original ) {
		$dimension = '';
	}

	if ( $gif ) {
		$ripple = 'ripple2.gif';
	}

	if ( ! strlen( $class ) > 0 ) {
		return;
	}

	echo '<img class="' . esc_html( $class ) . '"  id="' . esc_html( $id ) . '" style="display:none;' . esc_html( $css ) . '" src="' . esc_url( ETCPF_URL ) . '/images/' . esc_html( $ripple ) . '"' . esc_html( $dimension ) . '/>';
}

function etcpf_get_biggif_loader( $class, $css = array() ) {
	$style = '';
	if ( count( $css ) > 0 ) {
		foreach ( $css as $key => $value ) {
			$style .= $key . ':' . $value . ';';
		}
	}
	if ( ! strlen( $class ) > 0 ) {
		return;
	}

	echo '<img class="' . esc_html( $class ) . '" style="' . esc_html( $style ) . '" src="' . esc_url( ETCPF_URL ) . '/images/bigload.gif' . '"/>';
}

function etcpf_update_all_abstract() {
	etcpf_update_all( false, false );
}

function etcpf_update_all( $feedids, $doRegCheck = true ) {
	require_once 'etsy-export-feed-wpincludes.php'; // The rest of the required-files moved here
	require_once 'core/data/savedfeed.php';

	$reg = new ETCPF_EtsyValidation();

	if ( $doRegCheck == true ) {
		if ( is_object( $reg->results ) && $reg->results->status != 'Active' ) {
			$dir        = wp_upload_dir();
			$upload_die = $dir['basedir'];
			$myfile = fopen( $upload_die . '/etsy.txt', 'w' ) or die( 'Unable to open file!' );
			$txt = json_encode( $reg );
			fwrite( $myfile, $txt );
			fclose( $myfile );

			return;
		}

		if ( is_array( $reg->results ) && $reg->results['status'] != 'Active' ) {
			$dir        = wp_upload_dir();
			$upload_die = $dir['basedir'];
			$myfile = fopen( $upload_die . '/etsy.txt', 'w' ) or die( 'Unable to open file!' );
			$txt = json_encode( $reg );
			fwrite( $myfile, $txt );
			fclose( $myfile );

			return;
		}
	}
	do_action( 'load_etcpf_modifiers' );
	add_action( 'etcpf_feed_main_hook', 'etcpf_update_all_step_2' );
	do_action( 'etcpf_feed_main_hook', $feedids );

	// to upload images
	add_action( 'etcpf_mutipl_images_upload', 'etcpf_doImageThingy' );
	do_action( 'etcpf_multipl_images_upload' );
}

function etcpf_doImageThingy() {
	require_once 'etsy-export-feed-wpincludes.php';
	$etsy = new ETCPF_Etsy();
	$etsy->upload_additional_images();
}

function etcpf_update_all_step_2( $feedIds ) {
	global $wpdb;
	$feed_table = $wpdb->prefix . 'etcpf_feeds';
	$where      = '';
	if ( $feedIds ) {
		$where = ' WHERE id =' . $feedIds;
		$limit = '';
	} else {
		$limit = ' ORDER BY updated_at ASC';
	}
	$sql              = 'SELECT id, type, filename,product_count FROM ' . $feed_table . $where . $limit;
	$feed_ids         = $wpdb->get_results( $sql );
	$savedProductList = null;
	// ***********************************************************
	// Build stack of aggregate providers
	// ***********************************************************
	$aggregateProviders = array();
	// ***********************************************************
	// Main
	// ***********************************************************
	foreach ( $feed_ids as $index => $this_feed_id ) {
		$saved_feed = new ETCPF_SavedFeed( $this_feed_id->id );
		// Make sure someone exists in the core who can provide the feed
		$providerFile = 'core/feeds/etsy/feed.php';
		if ( ! file_exists( dirname( __FILE__ ) . '/' . $providerFile ) ) {
			continue;
		}
		require_once $providerFile;
		// Initialize provider data
		$x              = new ETCPF_EtsyFeed();
		$x->savedFeedID = $saved_feed->id;

		$catpath              = $saved_feed->category_path;
		$taxonomy_path        = $saved_feed->texonomy_path;
		$x->productList       = $savedProductList;
		$shipping_template_id = $wpdb->get_var( "SELECT SHIPPING_TEMPLATE_ID from $feed_table WHERE id = $this_feed_id->id" );

		if ( $saved_feed->feed_type <= 0 ) {
			$x->getFeedData( $saved_feed->category_id, $saved_feed->remote_category, $saved_feed->filename, $shipping_template_id, $saved_feed, $catpath, $taxonomy_path, true );
			$savedProductList = $x->productList;
			$x->products      = null;
		} else {
			include_once 'core/classes/ETCPF_Customfeed.php';
			$CustomObj = new ETCPF_Customfeed( 1 );
			$params    = array(
				'filename'   => $saved_feed->filename,
				'categories' => $CustomObj->getMappedCategoryOfParticularFeed( $this_feed_id->id ),
				'products'   => $CustomObj->getProductsOfParticularFeed( $this_feed_id->id ),
				'feedtype'   => 1,
			);
			$x->updateCustomFeed( $this_feed_id->id, $params, $saved_feed );
		}

		$listingtable = $wpdb->prefix . 'etcpf_listings';
		$check        = $wpdb->get_results( $wpdb->prepare( "SELECT count(id) as count FROM $listingtable WHERE uploaded=%d AND feed_id=%d", array(
			0,
			$this_feed_id->id
		) ) );
		if ( intval( $check[0]->count ) > 0 ) {
			// error_log("came to count. count is:".json_encode($check));
			continue;
		} else {
			include_once 'core/classes/etsy-upload.php';
			// error_log("came to preparing listing for id {$this_feed_id->id}");
			$prepareData = new ETCPF_EtsyUpload();
			$prepareData->set_shipping_template_id( $shipping_template_id );
			$prepareData->prepare_the_list_from_feed( $this_feed_id->id, true );
			update_option( 'currently_uploading_feed_id', 0 );
		}
	}

}

// ***********************************************************
// Links From the Install Plugins Page (WordPress)
// ***********************************************************

if ( is_admin() ) {
	require_once 'etsy-export-feed-admin.php';
	$plugin_ = plugin_basename( __FILE__ );
	add_filter( 'plugin_action_links_' . $plugin_, 'etcpf_manage_feed' );
}
// ***********************************************************
// Function to create feed generation link  in installed plugin page
// ***********************************************************
function etcpf_manage_feed( $links ) {
	$settings_link = '<a href="admin.php?page=etsy-export-feed-manage">Manage Feeds</a>';
	array_unshift( $links, $settings_link );

	return $links;
}


function etcpf_auto_feed_submission() {
	include_once 'core/classes/Etsy_auto_uploader.php';
	$updater = new ETCPF_Etsyupdater();
	$updater->uploadtoetsy();
}

function etcpf_auto_fetch_product() {
	include_once 'core/classes/ETCPF_Etsyorder.php';
	$updater = new ETCPF_Etsyorder();
	$updater->countEtsyProduct();
}

function etcpf_auto_etsy_order() {
	include_once 'core/classes/ETCPF_Etsyorder.php';
	$offset  = get_option( 'etsy_order_offset', $default = 0 );
	$updater = new ETCPF_Etsyorder();
	$updater->fetch_etsy_orders( etcpf_get_etsy_settings( 'order_fetching_time_interval' ), $offset );
	do_action( 'etcpf_update_all_step_2' );
}

add_action( 'woocommerce_update_product', 'etcpf_mp_sync_on_product_save', 10, 1 );
function etcpf_mp_sync_on_product_save( $product_id ) {
	if ( ! $product_id ) {
		return false;
	}
	if ( isset( $_POST['action_performed'] ) && ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['action_performed'] ) ), 'exportfeed_etsy_cpf' ) ) {
		return false;
	}

	$_POST['action_performed'] = true;
	require_once 'core/classes/product_uploadhook.php';
	$invoker = new ETCPF_Product_uploadhoook( $product_id );
	$invoker->Invoker();
	$_POST['action_performed'] = true;
}
